@@ -21,6 +21,7 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/syscall.h>
@@ -31,6 +32,10 @@
 
 #define SYS_IOCTL(fd, cmd, arg) \
 	syscall(SYS_ioctl, (int)(fd), (unsigned long)(cmd), (void *)(arg))
+#define SYS_READ(fd, buf, len) \
+	syscall(SYS_read, (int)(fd), (void *)(buf), (size_t)(len));
+#define SYS_WRITE(fd, buf, len) \
+	syscall(SYS_write, (int)(fd), (const void *)(buf), (size_t)(len));
 
 
 #if HAVE_VISIBILITY
@@ -53,9 +58,15 @@ struct mplane_plugin {
 	int __ret;						\
 	struct __struc *req = arg;				\
 	uint32_t type = req->type;				\
-	req->type = convert_type(type);				\
-	__ret = SYS_IOCTL(fd, cmd, arg);			\
-	req->type = type;					\
+	if (type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE ||	\
+	    type == V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE) {	\
+		errno = EINVAL;					\
+		__ret = -1;					\
+	} else {						\
+		req->type = convert_type(type);			\
+		__ret = SYS_IOCTL(fd, cmd, arg);		\
+		req->type = type;				\
+	}							\
 	__ret;							\
 	})
 
@@ -126,12 +137,23 @@ static int querycap_ioctl(int fd, unsigned long int cmd,
 	if (cap->capabilities & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
 		cap->capabilities |= V4L2_CAP_VIDEO_OUTPUT;
 
+	if (cap->device_caps & V4L2_CAP_VIDEO_CAPTURE_MPLANE)
+		cap->device_caps |= V4L2_CAP_VIDEO_CAPTURE;
+
+	if (cap->device_caps & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
+		cap->device_caps |= V4L2_CAP_VIDEO_OUTPUT;
+
+	cap->capabilities |= V4L2_CAP_EXT_PIX_FORMAT;
+	cap->device_caps |= V4L2_CAP_EXT_PIX_FORMAT;
+
 	/*
 	 * Don't report mplane caps, as this will be handled via
 	 * this plugin
 	 */
 	cap->capabilities &= ~(V4L2_CAP_VIDEO_OUTPUT_MPLANE |
 			       V4L2_CAP_VIDEO_CAPTURE_MPLANE);
+	cap->device_caps &= ~(V4L2_CAP_VIDEO_OUTPUT_MPLANE |
+			      V4L2_CAP_VIDEO_CAPTURE_MPLANE);
 
 	return 0;
 }
@@ -148,6 +170,32 @@ static int convert_type(int type)
 	}
 }
 
+static void sanitize_format(struct v4l2_format *fmt)
+{
+	unsigned int offset;
+
+	/*
+	 * The v4l2_pix_format structure has been extended with fields that were
+	 * not previously required to be set to zero by applications. The priv
+	 * field, when set to a magic value, indicates the the extended fields
+	 * are valid. We support these extended fields since struct
+	 * v4l2_pix_format_mplane supports those fields as well.
+	 *
+	 * So this function will sanitize v4l2_pix_format if priv != PRIV_MAGIC
+	 * by setting priv to that value and zeroing the remaining fields.
+	 */
+
+	if (fmt->fmt.pix.priv == V4L2_PIX_FMT_PRIV_MAGIC)
+		return;
+
+	fmt->fmt.pix.priv = V4L2_PIX_FMT_PRIV_MAGIC;
+
+	offset = offsetof(struct v4l2_pix_format, priv)
+	       + sizeof(fmt->fmt.pix.priv);
+	memset(((char *)&fmt->fmt.pix) + offset, 0,
+	       sizeof(fmt->fmt.pix) - offset);
+}
+
 static int try_set_fmt_ioctl(int fd, unsigned long int cmd,
 			     struct v4l2_format *arg)
 {
@@ -162,16 +210,23 @@ static int try_set_fmt_ioctl(int fd, unsigned long int cmd,
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
 		fmt.type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
 		break;
+	case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
+	case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
+		errno = EINVAL;
+		return -1;
 	default:
-		return SYS_IOCTL(fd, cmd, &fmt);
+		return SYS_IOCTL(fd, cmd, arg);
 	}
 
+	sanitize_format(org);
+
 	fmt.fmt.pix_mp.width = org->fmt.pix.width;
 	fmt.fmt.pix_mp.height = org->fmt.pix.height;
 	fmt.fmt.pix_mp.pixelformat = org->fmt.pix.pixelformat;
 	fmt.fmt.pix_mp.field = org->fmt.pix.field;
 	fmt.fmt.pix_mp.colorspace = org->fmt.pix.colorspace;
 	fmt.fmt.pix_mp.num_planes = 1;
+	fmt.fmt.pix_mp.flags = org->fmt.pix.flags;
 	fmt.fmt.pix_mp.plane_fmt[0].bytesperline = org->fmt.pix.bytesperline;
 	fmt.fmt.pix_mp.plane_fmt[0].sizeimage = org->fmt.pix.sizeimage;
 
@@ -186,6 +241,7 @@ static int try_set_fmt_ioctl(int fd, unsigned long int cmd,
 	org->fmt.pix.colorspace = fmt.fmt.pix_mp.colorspace;
 	org->fmt.pix.bytesperline = fmt.fmt.pix_mp.plane_fmt[0].bytesperline;
 	org->fmt.pix.sizeimage = fmt.fmt.pix_mp.plane_fmt[0].sizeimage;
+	org->fmt.pix.flags = fmt.fmt.pix_mp.flags;
 
 	return 0;
 }
@@ -193,43 +249,54 @@ static int try_set_fmt_ioctl(int fd, unsigned long int cmd,
 static int create_bufs_ioctl(int fd, unsigned long int cmd,
 			     struct v4l2_create_buffers *arg)
 {
-	struct v4l2_format fmt = { 0 };
+	struct v4l2_create_buffers cbufs = { 0 };
+	struct v4l2_format *fmt = &cbufs.format;
 	struct v4l2_format *org = &arg->format;
 	int ret;
 
 	switch (arg->format.type) {
 	case V4L2_BUF_TYPE_VIDEO_CAPTURE:
-		fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE;
+		fmt->type = V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE;
 		break;
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
-		fmt.type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
+		fmt->type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
 		break;
+	case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
+	case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
+		errno = EINVAL;
+		return -1;
 	default:
-		return SYS_IOCTL(fd, cmd, &fmt);
+		return SYS_IOCTL(fd, cmd, arg);
 	}
 
-	fmt.fmt.pix_mp.width = org->fmt.pix.width;
-	fmt.fmt.pix_mp.height = org->fmt.pix.height;
-	fmt.fmt.pix_mp.pixelformat = org->fmt.pix.pixelformat;
-	fmt.fmt.pix_mp.field = org->fmt.pix.field;
-	fmt.fmt.pix_mp.colorspace = org->fmt.pix.colorspace;
-	fmt.fmt.pix_mp.num_planes = 1;
-	fmt.fmt.pix_mp.plane_fmt[0].bytesperline = org->fmt.pix.bytesperline;
-	fmt.fmt.pix_mp.plane_fmt[0].sizeimage = org->fmt.pix.sizeimage;
+	cbufs.index = arg->index;
+	cbufs.count = arg->count;
+	cbufs.memory = arg->memory;
+	sanitize_format(org);
+	fmt->fmt.pix_mp.width = org->fmt.pix.width;
+	fmt->fmt.pix_mp.height = org->fmt.pix.height;
+	fmt->fmt.pix_mp.pixelformat = org->fmt.pix.pixelformat;
+	fmt->fmt.pix_mp.field = org->fmt.pix.field;
+	fmt->fmt.pix_mp.colorspace = org->fmt.pix.colorspace;
+	fmt->fmt.pix_mp.num_planes = 1;
+	fmt->fmt.pix_mp.flags = org->fmt.pix.flags;
+	fmt->fmt.pix_mp.plane_fmt[0].bytesperline = org->fmt.pix.bytesperline;
+	fmt->fmt.pix_mp.plane_fmt[0].sizeimage = org->fmt.pix.sizeimage;
 
-	ret = SYS_IOCTL(fd, cmd, &arg);
-	if (ret)
-		return ret;
+	ret = SYS_IOCTL(fd, cmd, &cbufs);
 
-	org->fmt.pix.width = fmt.fmt.pix_mp.width;
-	org->fmt.pix.height = fmt.fmt.pix_mp.height;
-	org->fmt.pix.pixelformat = fmt.fmt.pix_mp.pixelformat;
-	org->fmt.pix.field = fmt.fmt.pix_mp.field;
-	org->fmt.pix.colorspace = fmt.fmt.pix_mp.colorspace;
-	org->fmt.pix.bytesperline = fmt.fmt.pix_mp.plane_fmt[0].bytesperline;
-	org->fmt.pix.sizeimage = fmt.fmt.pix_mp.plane_fmt[0].sizeimage;
+	arg->index = cbufs.index;
+	arg->count = cbufs.count;
+	org->fmt.pix.width = fmt->fmt.pix_mp.width;
+	org->fmt.pix.height = fmt->fmt.pix_mp.height;
+	org->fmt.pix.pixelformat = fmt->fmt.pix_mp.pixelformat;
+	org->fmt.pix.field = fmt->fmt.pix_mp.field;
+	org->fmt.pix.colorspace = fmt->fmt.pix_mp.colorspace;
+	org->fmt.pix.bytesperline = fmt->fmt.pix_mp.plane_fmt[0].bytesperline;
+	org->fmt.pix.sizeimage = fmt->fmt.pix_mp.plane_fmt[0].sizeimage;
+	org->fmt.pix.flags = fmt->fmt.pix_mp.flags;
 
-	return 0;
+	return ret;
 }
 
 static int get_fmt_ioctl(int fd, unsigned long int cmd, struct v4l2_format *arg)
@@ -245,14 +312,19 @@ static int get_fmt_ioctl(int fd, unsigned long int cmd, struct v4l2_format *arg)
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
 		fmt.type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
 		break;
+	case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
+	case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
+		errno = EINVAL;
+		return -1;
 	default:
-		return SYS_IOCTL(fd, cmd, &fmt);
+		return SYS_IOCTL(fd, cmd, arg);
 	}
 
 	ret = SYS_IOCTL(fd, cmd, &fmt);
 	if (ret)
 		return ret;
 
+	memset(&org->fmt.pix, 0, sizeof(org->fmt.pix));
 	org->fmt.pix.width = fmt.fmt.pix_mp.width;
 	org->fmt.pix.height = fmt.fmt.pix_mp.height;
 	org->fmt.pix.pixelformat = fmt.fmt.pix_mp.pixelformat;
@@ -260,13 +332,15 @@ static int get_fmt_ioctl(int fd, unsigned long int cmd, struct v4l2_format *arg)
 	org->fmt.pix.colorspace = fmt.fmt.pix_mp.colorspace;
 	org->fmt.pix.bytesperline = fmt.fmt.pix_mp.plane_fmt[0].bytesperline;
 	org->fmt.pix.sizeimage = fmt.fmt.pix_mp.plane_fmt[0].sizeimage;
+	org->fmt.pix.priv = V4L2_PIX_FMT_PRIV_MAGIC;
+	org->fmt.pix.flags = fmt.fmt.pix_mp.flags;
 
 	/*
 	 * If the device doesn't support just one plane, there's
 	 * nothing we can do, except return an error condition.
 	 */
 	if (fmt.fmt.pix_mp.num_planes > 1) {
-		errno = -EINVAL;
+		errno = EINVAL;
 		return -1;
 	}
 
@@ -279,6 +353,12 @@ static int buf_ioctl(int fd, unsigned long int cmd, struct v4l2_buffer *arg)
 	struct v4l2_plane plane = { 0 };
 	int ret;
 
+	if (arg->type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE ||
+	    arg->type == V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE) {
+		errno = EINVAL;
+		return -1;
+	}
+
 	buf.type = convert_type(arg->type);
 
 	if (buf.type == arg->type)
@@ -294,6 +374,7 @@ static int buf_ioctl(int fd, unsigned long int cmd, struct v4l2_buffer *arg)
 	ret = SYS_IOCTL(fd, cmd, &buf);
 
 	arg->index = buf.index;
+	arg->memory = buf.memory;
 	arg->flags = buf.flags;
 	arg->field = buf.field;
 	arg->timestamp = buf.timestamp;
@@ -302,6 +383,7 @@ static int buf_ioctl(int fd, unsigned long int cmd, struct v4l2_buffer *arg)
 
 	arg->length = plane.length;
 	arg->bytesused = plane.bytesused;
+	memcpy(&arg->m, &plane.m, sizeof(arg->m));
 
 	return ret;
 }
@@ -309,7 +391,6 @@ static int buf_ioctl(int fd, unsigned long int cmd, struct v4l2_buffer *arg)
 static int plugin_ioctl(void *dev_ops_priv, int fd,
 			unsigned long int cmd, void *arg)
 {
-	struct mplane_plugin *plugin = dev_ops_priv;
 	switch (cmd) {
 	case VIDIOC_QUERYCAP:
 		return querycap_ioctl(fd, cmd, arg);
@@ -323,11 +404,6 @@ static int plugin_ioctl(void *dev_ops_priv, int fd,
 	case VIDIOC_S_PARM:
 	case VIDIOC_G_PARM:
 		return SIMPLE_CONVERT_IOCTL(fd, cmd, arg, v4l2_streamparm);
-	case VIDIOC_CROPCAP:
-		return SIMPLE_CONVERT_IOCTL(fd, cmd, arg, v4l2_cropcap);
-	case VIDIOC_S_CROP:
-	case VIDIOC_G_CROP:
-		return SIMPLE_CONVERT_IOCTL(fd, cmd, arg, v4l2_crop);
 	case VIDIOC_QBUF:
 	case VIDIOC_DQBUF:
 	case VIDIOC_QUERYBUF:
@@ -340,26 +416,16 @@ static int plugin_ioctl(void *dev_ops_priv, int fd,
 	case VIDIOC_STREAMON:
 	case VIDIOC_STREAMOFF:
 	{
-		int type, ret;
+		int type = *(int *)arg;
 
-		/*
-		 * If the device has both capture and output, weird things
-		 * could happen. For now, let's not consider this case. If this
-		 * is ever happens in practice, the logic should be changed to
-		 * track reqbufs, in order to identify what's required.
-		 */
-		if (plugin->mplane_capture) {
-			type = V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE;
-			ret = SYS_IOCTL(fd, cmd, &type);
-		} else if (plugin->mplane_output) {
-			type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
-			ret = SYS_IOCTL(fd, cmd, &type);
-		} else {
-			ret = -1;
+		if (type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE ||
+		    type == V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE) {
 			errno = EINVAL;
+			return -1;
 		}
+		type = convert_type(type);
 
-		return ret;
+		return SYS_IOCTL(fd, cmd, &type);
 	}
 	/* CASE VIDIOC_EXPBUF: */
 	default:
@@ -367,8 +433,21 @@ static int plugin_ioctl(void *dev_ops_priv, int fd,
 	}
 }
 
+static ssize_t plugin_read(void *dev_ops_priv, int fd, void *buf, size_t len)
+{
+	return SYS_READ(fd, buf, len);
+}
+
+static ssize_t plugin_write(void *dev_ops_priv, int fd, const void *buf,
+                         size_t len)
+{
+	return SYS_WRITE(fd, buf, len);
+}
+
 PLUGIN_PUBLIC const struct libv4l_dev_ops libv4l2_plugin = {
 	.init = &plugin_init,
 	.close = &plugin_close,
 	.ioctl = &plugin_ioctl,
+	.read = &plugin_read,
+	.write = &plugin_write,
 };
