@@ -84,6 +84,8 @@
 #define V4L2_MMAP_OFFSET_MAGIC      0xABCDEF00u
 
 static void v4l2_adjust_src_fmt_to_fps(int index, int fps);
+static void v4l2_set_src_and_dest_format(int index,
+		struct v4l2_format *src_fmt, struct v4l2_format *dest_fmt);
 
 static pthread_mutex_t v4l2_open_mutex = PTHREAD_MUTEX_INITIALIZER;
 static struct v4l2_dev_info devices[V4L2_MAX_DEVICES] = {
@@ -94,6 +96,32 @@ static struct v4l2_dev_info devices[V4L2_MAX_DEVICES] = {
 };
 static int devices_used;
 
+static int v4l2_ensure_convert_mmap_buf(int index)
+{
+	if (devices[index].convert_mmap_buf != MAP_FAILED) {
+		return 0;
+	}
+
+	devices[index].convert_mmap_buf_size =
+		devices[index].convert_mmap_frame_size * devices[index].no_frames;
+
+	devices[index].convert_mmap_buf = (void *)SYS_MMAP(NULL,
+			devices[index].convert_mmap_buf_size,
+			PROT_READ | PROT_WRITE,
+			MAP_ANONYMOUS | MAP_PRIVATE,
+			-1, 0);
+
+	if (devices[index].convert_mmap_buf == MAP_FAILED) {
+		devices[index].convert_mmap_buf_size = 0;
+
+		int saved_err = errno;
+		V4L2_LOG_ERR("allocating conversion buffer\n");
+		errno = saved_err;
+		return -1;
+	}
+
+	return 0;
+}
 
 static int v4l2_request_read_buffers(int index)
 {
@@ -158,6 +186,7 @@ static int v4l2_map_buffers(int index)
 		buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 		buf.memory = V4L2_MEMORY_MMAP;
 		buf.index = i;
+		buf.reserved = buf.reserved2 = 0;
 		result = devices[index].dev_ops->ioctl(
 				devices[index].dev_ops_priv,
 				devices[index].fd, VIDIOC_QUERYBUF, &buf);
@@ -317,7 +346,8 @@ static int v4l2_dequeue_and_convert(int index, struct v4l2_buffer *buf,
 				&devices[index].src_fmt, &devices[index].dest_fmt,
 				devices[index].frame_pointers[buf->index],
 				buf->bytesused, dest ? dest : (devices[index].convert_mmap_buf +
-					buf->index * V4L2_FRAME_BUF_SIZE), dest_size);
+					buf->index * devices[index].convert_mmap_frame_size),
+				dest_size);
 
 		if (devices[index].first_frame) {
 			/* Always treat convert errors as EAGAIN during the first few frames, as
@@ -526,7 +556,7 @@ static void v4l2_set_conversion_buf_params(int index, struct v4l2_buffer *buf)
 		buf->index = 0;
 
 	buf->m.offset = V4L2_MMAP_OFFSET_MAGIC | buf->index;
-	buf->length = V4L2_FRAME_BUF_SIZE;
+	buf->length = devices[index].convert_mmap_frame_size;
 	if (devices[index].frame_map_count[buf->index])
 		buf->flags |= V4L2_BUF_FLAG_MAPPED;
 	else
@@ -545,6 +575,7 @@ static int v4l2_buffers_mapped(int index)
 			buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 			buf.memory = V4L2_MEMORY_MMAP;
 			buf.index = i;
+			buf.reserved = buf.reserved2 = 0;
 			if (devices[index].dev_ops->ioctl(
 					devices[index].dev_ops_priv,
 					devices[index].fd, VIDIOC_QUERYBUF,
@@ -628,6 +659,7 @@ int v4l2_fd_open(int fd, int v4l2_flags)
 	void *plugin_library;
 	void *dev_ops_priv;
 	const struct libv4l_dev_ops *dev_ops;
+	long page_size;
 
 	v4l2_plugin_init(fd, &plugin_library, &dev_ops_priv, &dev_ops);
 
@@ -639,6 +671,17 @@ int v4l2_fd_open(int fd, int v4l2_flags)
 			v4l2_log_file = fopen(lfname, "w");
 	}
 
+	/* Get page_size (for mmap emulation) */
+	page_size = sysconf(_SC_PAGESIZE);
+	if (page_size < 0) {
+		int saved_err = errno;
+		V4L2_LOG_ERR("unable to retrieve page size: %s\n",
+			     strerror(errno));
+		v4l2_plugin_cleanup(plugin_library, dev_ops_priv, dev_ops);
+		errno = saved_err;
+		return -1;
+	}
+
 	/* check that this is a v4l2 device */
 	if (dev_ops->ioctl(dev_ops_priv, fd, VIDIOC_QUERYCAP, &cap)) {
 		int saved_err = errno;
@@ -716,18 +759,11 @@ no_capture:
 	    (parm.parm.capture.capability & V4L2_CAP_TIMEPERFRAME))
 		devices[index].flags |= V4L2_SUPPORTS_TIMEPERFRAME;
 	devices[index].open_count = 1;
-	devices[index].src_fmt = fmt;
+	devices[index].page_size = page_size;
+	devices[index].src_fmt  = fmt;
 	devices[index].dest_fmt = fmt;
-
-	/* When a user does a try_fmt with the current dest_fmt and the dest_fmt
-	   is a supported one we will align the resolution (see try_fmt for why).
-	   Do the same here now, so that a try_fmt on the result of a get_fmt done
-	   immediately after open leaves the fmt unchanged. */
-	if (v4lconvert_supported_dst_format(
-				devices[index].dest_fmt.fmt.pix.pixelformat)) {
-		devices[index].dest_fmt.fmt.pix.width &= ~7;
-		devices[index].dest_fmt.fmt.pix.height &= ~1;
-	}
+	v4l2_set_src_and_dest_format(index, &devices[index].src_fmt,
+				     &devices[index].dest_fmt);
 
 	pthread_mutex_init(&devices[index].stream_lock, NULL);
 
@@ -735,6 +771,7 @@ no_capture:
 	devices[index].nreadbuffers = V4L2_DEFAULT_NREADBUFFERS;
 	devices[index].convert = convert;
 	devices[index].convert_mmap_buf = MAP_FAILED;
+	devices[index].convert_mmap_buf_size = 0;
 	for (i = 0; i < V4L2_MAX_NO_FRAMES; i++) {
 		devices[index].frame_pointers[i] = MAP_FAILED;
 		devices[index].frame_map_count[i] = 0;
@@ -808,9 +845,10 @@ int v4l2_close(int fd)
 				V4L2_LOG_WARN("v4l2 mmap buffers still mapped on close()\n");
 		} else {
 			SYS_MUNMAP(devices[index].convert_mmap_buf,
-					devices[index].no_frames * V4L2_FRAME_BUF_SIZE);
+					devices[index].convert_mmap_buf_size);
 		}
 		devices[index].convert_mmap_buf = MAP_FAILED;
+		devices[index].convert_mmap_buf_size = 0;
 	}
 	v4lconvert_destroy(devices[index].convert);
 	free(devices[index].readbuf);
@@ -864,8 +902,9 @@ static int v4l2_check_buffer_change_ok(int index)
 	   v4l2_unrequest_read_buffers may change the no_frames, so free the
 	   convert mmap buffer */
 	SYS_MUNMAP(devices[index].convert_mmap_buf,
-			devices[index].no_frames * V4L2_FRAME_BUF_SIZE);
+			devices[index].convert_mmap_buf_size);
 	devices[index].convert_mmap_buf = MAP_FAILED;
+	devices[index].convert_mmap_buf_size = 0;
 
 	if (devices[index].flags & V4L2_STREAM_CONTROLLED_BY_READ) {
 		V4L2_LOG("deactivating read-stream for settings change\n");
@@ -899,6 +938,18 @@ static int v4l2_pix_fmt_identical(struct v4l2_format *a, struct v4l2_format *b)
 static void v4l2_set_src_and_dest_format(int index,
 		struct v4l2_format *src_fmt, struct v4l2_format *dest_fmt)
 {
+	/*
+	 * When a user does a try_fmt with the current dest_fmt and the
+	 * dest_fmt is a supported one we will align the resolution (see
+	 * libv4lconvert_try_fmt). We do this here too, in case dest_fmt gets
+	 * set without having gone through libv4lconvert_try_fmt, so that a
+	 * try_fmt on the result of a get_fmt always returns the same result.
+	 */
+	if (v4lconvert_supported_dst_format(dest_fmt->fmt.pix.pixelformat)) {
+		dest_fmt->fmt.pix.width &= ~7;
+		dest_fmt->fmt.pix.height &= ~1;
+	}
+
 	/* Sigh some drivers (pwc) do not properly reflect what one really gets
 	   after a s_fmt in their try_fmt answer. So update dest format (which we
 	   report as result from s_fmt / g_fmt to the app) with all info from the src
@@ -909,10 +960,15 @@ static void v4l2_set_src_and_dest_format(int index,
 	if (v4l2_pix_fmt_compat(src_fmt, dest_fmt)) {
 		dest_fmt->fmt.pix.bytesperline = src_fmt->fmt.pix.bytesperline;
 		dest_fmt->fmt.pix.sizeimage = src_fmt->fmt.pix.sizeimage;
-	}
+	} else
+		v4lconvert_fixup_fmt(dest_fmt);
 
 	devices[index].src_fmt = *src_fmt;
 	devices[index].dest_fmt = *dest_fmt;
+	/* round up to full page size */
+	devices[index].convert_mmap_frame_size =
+		(((dest_fmt->fmt.pix.sizeimage + devices[index].page_size - 1)
+		/ devices[index].page_size) * devices[index].page_size);
 }
 
 static int v4l2_s_fmt(int index, struct v4l2_format *dest_fmt)
@@ -1022,6 +1078,9 @@ int v4l2_ioctl(int fd, unsigned long int request, ...)
 	case VIDIOC_QUERYCTRL:
 	case VIDIOC_G_CTRL:
 	case VIDIOC_S_CTRL:
+	case VIDIOC_G_EXT_CTRLS:
+	case VIDIOC_TRY_EXT_CTRLS:
+	case VIDIOC_S_EXT_CTRLS:
 	case VIDIOC_ENUM_FRAMESIZES:
 	case VIDIOC_ENUM_FRAMEINTERVALS:
 		is_capture_request = 1;
@@ -1129,6 +1188,18 @@ no_capture_request:
 		result = v4lconvert_vidioc_s_ctrl(devices[index].convert, arg);
 		break;
 
+	case VIDIOC_G_EXT_CTRLS:
+		result = v4lconvert_vidioc_g_ext_ctrls(devices[index].convert, arg);
+		break;
+
+	case VIDIOC_TRY_EXT_CTRLS:
+		result = v4lconvert_vidioc_try_ext_ctrls(devices[index].convert, arg);
+		break;
+
+	case VIDIOC_S_EXT_CTRLS:
+		result = v4lconvert_vidioc_s_ext_ctrls(devices[index].convert, arg);
+		break;
+
 	case VIDIOC_QUERYCAP: {
 		struct v4l2_capability *cap = arg;
 
@@ -1178,7 +1249,9 @@ no_capture_request:
 	case VIDIOC_S_STD:
 	case VIDIOC_S_INPUT:
 	case VIDIOC_S_DV_TIMINGS: {
-		struct v4l2_format src_fmt;
+		struct v4l2_format src_fmt = { 0 };
+		unsigned int orig_dest_pixelformat =
+			devices[index].dest_fmt.fmt.pix.pixelformat;
 
 		result = devices[index].dev_ops->ioctl(
 				devices[index].dev_ops_priv,
@@ -1207,9 +1280,10 @@ no_capture_request:
 		/* The fmt has been changed, remember the new format ... */
 		devices[index].src_fmt  = src_fmt;
 		devices[index].dest_fmt = src_fmt;
+		v4l2_set_src_and_dest_format(index, &devices[index].src_fmt,
+					     &devices[index].dest_fmt);
 		/* and try to restore the last set destination pixelformat. */
-		src_fmt.fmt.pix.pixelformat =
-			devices[index].dest_fmt.fmt.pix.pixelformat;
+		src_fmt.fmt.pix.pixelformat = orig_dest_pixelformat;
 		result = v4l2_s_fmt(index, &src_fmt);
 		if (result) {
 			V4L2_LOG_WARN("restoring destination pixelformat after %s failed\n",
@@ -1319,23 +1393,12 @@ no_capture_request:
 		/* An application can do a DQBUF before mmap-ing in the buffer,
 		   but we need the buffer _now_ to write our converted data
 		   to it! */
-		if (devices[index].convert_mmap_buf == MAP_FAILED) {
-			devices[index].convert_mmap_buf = (void *)SYS_MMAP(NULL,
-				(size_t)(devices[index].no_frames * V4L2_FRAME_BUF_SIZE),
-				PROT_READ | PROT_WRITE,
-				MAP_ANONYMOUS | MAP_PRIVATE,
-				-1, 0);
-			if (devices[index].convert_mmap_buf == MAP_FAILED) {
-				saved_err = errno;
-				V4L2_LOG_ERR("allocating conversion buffer\n");
-				errno = saved_err;
-				result = -1;
-				break;
-			}
-		}
+		result = v4l2_ensure_convert_mmap_buf(index);
+		if (result)
+			break;
 
 		result = v4l2_dequeue_and_convert(index, buf, 0,
-						  V4L2_FRAME_BUF_SIZE);
+				devices[index].convert_mmap_frame_size);
 		if (result >= 0) {
 			buf->bytesused = result;
 			result = 0;
@@ -1560,7 +1623,7 @@ void *v4l2_mmap(void *start, size_t length, int prot, int flags, int fd,
 	if (index == -1 ||
 			/* Check if the mmap data matches our answer to QUERY_BUF. If it doesn't,
 			   let the kernel handle it (to allow for mmap-based non capture use) */
-			start || length != V4L2_FRAME_BUF_SIZE ||
+			start || length != devices[index].convert_mmap_frame_size ||
 			((unsigned int)offset & ~0xFFu) != V4L2_MMAP_OFFSET_MAGIC) {
 		if (index != -1)
 			V4L2_LOG("Passing mmap(%p, %d, ..., %x, through to the driver\n",
@@ -1585,26 +1648,16 @@ void *v4l2_mmap(void *start, size_t length, int prot, int flags, int fd,
 		goto leave;
 	}
 
-	if (devices[index].convert_mmap_buf == MAP_FAILED) {
-		devices[index].convert_mmap_buf = (void *)SYS_MMAP(NULL,
-			(size_t)(devices[index].no_frames * V4L2_FRAME_BUF_SIZE),
-			PROT_READ | PROT_WRITE,
-			MAP_ANONYMOUS | MAP_PRIVATE,
-			-1, 0);
-		if (devices[index].convert_mmap_buf == MAP_FAILED) {
-			int saved_err = errno;
-
-			V4L2_LOG_ERR("allocating conversion buffer\n");
-			errno = saved_err;
-			result = MAP_FAILED;
-			goto leave;
-		}
+	if (v4l2_ensure_convert_mmap_buf(index)) {
+		errno = EINVAL;
+		result = MAP_FAILED;
+		goto leave;
 	}
 
 	devices[index].frame_map_count[buffer_index]++;
 
 	result = devices[index].convert_mmap_buf +
-		buffer_index * V4L2_FRAME_BUF_SIZE;
+		buffer_index * devices[index].convert_mmap_frame_size;
 
 	V4L2_LOG("Fake (conversion) mmap buf %u, seen by app at: %p\n",
 			buffer_index, result);
@@ -1622,10 +1675,11 @@ int v4l2_munmap(void *_start, size_t length)
 	unsigned char *start = _start;
 
 	/* Is this memory ours? */
-	if (start != MAP_FAILED && length == V4L2_FRAME_BUF_SIZE) {
+	if (start != MAP_FAILED) {
 		for (index = 0; index < devices_used; index++)
 			if (devices[index].fd != -1 &&
 					devices[index].convert_mmap_buf != MAP_FAILED &&
+					length == devices[index].convert_mmap_frame_size &&
 					start >= devices[index].convert_mmap_buf &&
 					(start - devices[index].convert_mmap_buf) % length == 0)
 				break;
@@ -1639,6 +1693,7 @@ int v4l2_munmap(void *_start, size_t length)
 
 			/* Re-do our checks now that we have the lock, things may have changed */
 			if (devices[index].convert_mmap_buf != MAP_FAILED &&
+					length == devices[index].convert_mmap_frame_size &&
 					start >= devices[index].convert_mmap_buf &&
 					(start - devices[index].convert_mmap_buf) % length == 0 &&
 					buffer_index < devices[index].no_frames) {
