@@ -86,6 +86,9 @@ enum ir_protocols {
 	LIRC		= 1 << 5,
 	SANYO		= 1 << 6,
 	RC_5_SZ		= 1 << 7,
+	SHARP		= 1 << 8,
+	MCE_KBD		= 1 << 9,
+	XMP		= 1 << 10,
 	OTHER		= 1 << 31,
 };
 
@@ -110,7 +113,8 @@ static const char doc[] = "\nAllows get/set IR keycode/scancode tables\n"
 	"  SYSDEV   - the ir class as found at /sys/class/rc\n"
 	"  TABLE    - a file with a set of scancode=keycode value pairs\n"
 	"  SCANKEY  - a set of scancode1=keycode1,scancode2=keycode2.. value pairs\n"
-	"  PROTOCOL - protocol name (nec, rc-5, rc-6, jvc, sony, sanyo, rc-5-sz, lirc, other) to be enabled\n"
+	"  PROTOCOL - protocol name (nec, rc-5, rc-6, jvc, sony, sanyo, rc-5-sz, lirc,\n"
+        "                            sharp, mce_kbd, xmp, other, all) to be enabled\n"
 	"  DELAY    - Delay before repeating a keystroke\n"
 	"  PERIOD   - Period to repeat a keystroke\n"
 	"  CFGFILE  - configuration file that associates a driver/table name with a keymap file\n"
@@ -234,6 +238,12 @@ static error_t parse_keyfile(char *fname, char **table)
 							ch_proto |= SANYO;
 						else if (!strcasecmp(p,"rc-5-sz"))
 							ch_proto |= RC_5_SZ;
+						else if (!strcasecmp(p,"sharp"))
+							ch_proto |= SHARP;
+						else if (!strcasecmp(p,"mce-kbd"))
+							ch_proto |= MCE_KBD;
+						else if (!strcasecmp(p,"xmp"))
+							ch_proto |= XMP;
 						else if (!strcasecmp(p,"other") || !strcasecmp(p,"unknown"))
 							ch_proto |= OTHER;
 						else {
@@ -471,6 +481,14 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 				ch_proto |= LIRC;
 			else if (!strcasecmp(p,"rc-5-sz"))
 				ch_proto |= RC_5_SZ;
+			else if (!strcasecmp(p,"sharp"))
+				ch_proto |= RC_5_SZ;
+			else if (!strcasecmp(p,"mce-kbd"))
+				ch_proto |= RC_5_SZ;
+			else if (!strcasecmp(p,"xmp"))
+				ch_proto |= XMP;
+			else if (!strcasecmp(p,"all"))
+				ch_proto |= ~0;
 			else
 				goto err_inval;
 			p = strtok(NULL, ",;");
@@ -744,6 +762,12 @@ static enum ir_protocols v1_get_hw_protocols(char *name)
 			proto |= SANYO;
 		else if (!strcmp(p, "rc-5-sz"))
 			proto |= RC_5_SZ;
+		else if (!strcmp(p, "sharp"))
+			proto |= SHARP;
+		else if (!strcmp(p, "mce-kbd"))
+			proto |= MCE_KBD;
+		else if (!strcmp(p, "xmp"))
+			proto |= XMP;
 		else
 			proto |= OTHER;
 
@@ -790,6 +814,15 @@ static int v1_set_hw_protocols(struct rc_device *rc_dev)
 	if (rc_dev->current & RC_5_SZ)
 		fprintf(fp, "rc-5-sz ");
 
+	if (rc_dev->current & SHARP)
+		fprintf(fp, "sharp ");
+
+	if (rc_dev->current & MCE_KBD)
+		fprintf(fp, "mce_kbd ");
+
+	if (rc_dev->current & XMP)
+		fprintf(fp, "xmp ");
+
 	if (rc_dev->current & OTHER)
 		fprintf(fp, "unknown ");
 
@@ -921,6 +954,12 @@ static enum ir_protocols v2_get_protocols(struct rc_device *rc_dev, char *name)
 			proto = LIRC;
 		else if (!strcmp(p, "rc-5-sz"))
 			proto = RC_5_SZ;
+		else if (!strcmp(p, "sharp"))
+			proto = SHARP;
+		else if (!strcmp(p, "mce-kbd"))
+			proto = MCE_KBD;
+		else if (!strcmp(p, "xmp"))
+			proto = XMP;
 		else
 			proto = OTHER;
 
@@ -977,6 +1016,15 @@ static int v2_set_protocols(struct rc_device *rc_dev)
 	if (rc_dev->current & RC_5_SZ)
 		fprintf(fp, "+rc-5-sz\n");
 
+	if (rc_dev->current & SHARP)
+		fprintf(fp, "+sharp\n");
+
+	if (rc_dev->current & MCE_KBD)
+		fprintf(fp, "+mce-kbd\n");
+
+	if (rc_dev->current & XMP)
+		fprintf(fp, "+xmp\n");
+
 	if (rc_dev->current & OTHER)
 		fprintf(fp, "+unknown\n");
 
@@ -1006,6 +1054,12 @@ static void show_proto(	enum ir_protocols proto)
 		fprintf (stderr, "LIRC ");
 	if (proto & RC_5_SZ)
 		fprintf (stderr, "RC-5-SZ ");
+	if (proto & SHARP)
+		fprintf (stderr, "SHARP ");
+	if (proto & MCE_KBD)
+		fprintf (stderr, "MCE_KBD ");
+	if (proto & XMP)
+		fprintf (stderr, "XMP ");
 	if (proto & OTHER)
 		fprintf (stderr, "other ");
 }
@@ -1128,6 +1182,10 @@ static int get_attribs(struct rc_device *rc_dev, char *sysfs_name)
 			rc_dev->supported |= SONY;
 			if (v1_get_sw_enabled_protocol(cur->name))
 				rc_dev->current |= SONY;
+		} else if (strstr(cur->name, "/xmp_decoder")) {
+			rc_dev->supported |= XMP;
+			if (v1_get_sw_enabled_protocol(cur->name))
+				rc_dev->current |= XMP;
 		}
 	}
 
@@ -1138,6 +1196,12 @@ static int set_proto(struct rc_device *rc_dev)
 {
 	int rc = 0;
 
+	rc_dev->current &= rc_dev->supported;
+	if (!rc_dev->current) {
+		fprintf(stderr, "Invalid protocols selected\n");
+		return EINVAL;
+	}
+
 	if (rc_dev->version == VERSION_2) {
 		rc = v2_set_protocols(rc_dev);
 		return rc;
@@ -1159,6 +1223,9 @@ static int set_proto(struct rc_device *rc_dev)
 		if (rc_dev->supported & SONY)
 			rc += v1_set_sw_enabled_protocol(rc_dev, "/sony_decoder",
 						      rc_dev->current & SONY);
+		if (rc_dev->supported & XMP)
+			rc += v1_set_sw_enabled_protocol(rc_dev, "/xmp_decoder",
+						      rc_dev->current & XMP);
 	} else {
 		rc = v1_set_hw_protocols(rc_dev);
 	}
