@@ -267,7 +267,7 @@ struct parse_event key_events[] = {
 	{"KEY_VIDEO_NEXT", 241},
 	{"KEY_VIDEO_PREV", 242},
 	{"KEY_BRIGHTNESS_CYCLE", 243},
-	{"KEY_BRIGHTNESS_ZERO", 244},
+	{"KEY_BRIGHTNESS_AUTO", 244},
 	{"KEY_DISPLAY_OFF", 245},
 	{"KEY_WWAN", 246},
 	{"KEY_RFKILL", 247},
@@ -499,6 +499,15 @@ struct parse_event key_events[] = {
 	{"BTN_DPAD_LEFT", 0x222},
 	{"BTN_DPAD_RIGHT", 0x223},
 	{"KEY_ALS_TOGGLE", 0x230},
+	{"KEY_BUTTONCONFIG", 0x240},
+	{"KEY_TASKMANAGER", 0x241},
+	{"KEY_JOURNAL", 0x242},
+	{"KEY_CONTROLPANEL", 0x243},
+	{"KEY_APPSELECT", 0x244},
+	{"KEY_SCREENSAVER", 0x245},
+	{"KEY_VOICECOMMAND", 0x246},
+	{"KEY_BRIGHTNESS_MIN", 0x250},
+	{"KEY_BRIGHTNESS_MAX", 0x251},
 	{"BTN_TRIGGER_HAPPY", 0x2c0},
 	{"BTN_TRIGGER_HAPPY1", 0x2c0},
 	{"BTN_TRIGGER_HAPPY2", 0x2c1},
