@@ -33,7 +33,7 @@
 #include <map>
 #include "v4l2-compliance.h"
 
-static struct v4l2_format cur_fmt;
+static struct cv4l_fmt cur_fmt;
 
 static const unsigned valid_output_flags =
 	V4L2_BUF_FLAG_TIMECODE | V4L2_BUF_FLAG_TSTAMP_SRC_MASK |
@@ -106,20 +106,8 @@ static std::string field2s(unsigned val)
 	}
 }
 
-class queue : public cv4l_queue {
-public:
-	queue(node *node, unsigned type = V4L2_BUF_TYPE_VIDEO_CAPTURE,
-			  unsigned memory = V4L2_MEMORY_MMAP) :
-		cv4l_queue(&node->vfd, type, memory) {}
-};
-
 class buffer : public cv4l_buffer {
 public:
-	using cv4l_buffer::querybuf;
-	using cv4l_buffer::prepare_buf;
-	using cv4l_buffer::qbuf;
-	using cv4l_buffer::dqbuf;
-
 	buffer(unsigned type = 0, unsigned memory = 0, unsigned index = 0) :
 		cv4l_buffer(type, memory, index) {}
 	buffer(const cv4l_queue &q, unsigned index = 0) :
@@ -128,39 +116,28 @@ public:
 
 	int querybuf(node *node, unsigned index)
 	{
-		return querybuf(&node->vfd, index);
+		return node->querybuf(*this, index);
 	}
 	int prepare_buf(node *node)
 	{
-		return prepare_buf(&node->vfd);
+		return node->prepare_buf(*this);
 	}
 	int dqbuf(node *node)
 	{
-		return dqbuf(&node->vfd);
+		return node->dqbuf(*this);
 	}
 	int qbuf(node *node, bool fill_bytesused = true)
 	{
 		int err;
 
-		if (is_output())
+		if (v4l_type_is_output(g_type()))
 			fill_output_buf(fill_bytesused);
-		err = qbuf(&node->vfd);
-		if (err == 0 && is_output())
+		err = node->qbuf(*this);
+		if (err == 0 && v4l_type_is_output(g_type()))
 			buffer_info[g_timestamp()] = buf;
 		return err;
 	}
-	int qbuf(const queue &q, bool fill_bytesused = true)
-	{
-		int err;
-
-		if (is_output())
-			fill_output_buf(fill_bytesused);
-		err = cv4l_buffer::qbuf(q);
-		if (err == 0 && is_output())
-			buffer_info[g_timestamp()] = buf;
-		return err;
-	}
-	int check(const queue &q, enum QueryBufMode mode)
+	int check(const cv4l_queue &q, enum QueryBufMode mode)
 	{
 		int ret = check(q.g_type(), q.g_memory(), g_index(), mode, last_seq);
 
@@ -168,7 +145,7 @@ public:
 			ret = check_planes(q, mode);
 		return ret;
 	}
-	int check(const queue &q, enum QueryBufMode mode, __u32 index)
+	int check(const cv4l_queue &q, enum QueryBufMode mode, __u32 index)
 	{
 		int ret = check(q.g_type(), q.g_memory(), index, mode, last_seq);
 
@@ -176,7 +153,7 @@ public:
 			ret = check_planes(q, mode);
 		return ret;
 	}
-	int check(const queue &q, buf_seq &seq)
+	int check(const cv4l_queue &q, buf_seq &seq)
 	{
 		int ret = check(q.g_type(), q.g_memory(), g_index(), Dequeued, seq);
 
@@ -196,7 +173,7 @@ public:
 private:
 	int check(unsigned type, unsigned memory, unsigned index,
 			enum QueryBufMode mode, struct buf_seq &seq);
-	int check_planes(const queue &q, enum QueryBufMode mode);
+	int check_planes(const cv4l_queue &q, enum QueryBufMode mode);
 	void fill_output_buf(bool fill_bytesused = true)
 	{
 		timespec ts;
@@ -229,7 +206,7 @@ private:
 	}
 };
 
-int buffer::check_planes(const queue &q, enum QueryBufMode mode)
+int buffer::check_planes(const cv4l_queue &q, enum QueryBufMode mode)
 {
 	if (mode == Dequeued || mode == Prepared) {
 		for (unsigned p = 0; p < g_num_planes(); p++) {
@@ -261,7 +238,7 @@ int buffer::check(unsigned type, unsigned memory, unsigned index,
 		     timestamp != V4L2_BUF_FLAG_TIMESTAMP_COPY);
 	fail_on_test(timestamp_src != V4L2_BUF_FLAG_TSTAMP_SRC_SOE &&
 		     timestamp_src != V4L2_BUF_FLAG_TSTAMP_SRC_EOF);
-	fail_on_test(!ts_copy && is_output() &&
+	fail_on_test(!ts_copy && v4l_type_is_output(g_type()) &&
 		     timestamp_src == V4L2_BUF_FLAG_TSTAMP_SRC_SOE);
 	if (g_flags() & V4L2_BUF_FLAG_KEYFRAME)
 		frame_types++;
@@ -276,13 +253,11 @@ int buffer::check(unsigned type, unsigned memory, unsigned index,
 		buf_states++;
 	if (g_flags() & V4L2_BUF_FLAG_DONE)
 		buf_states++;
-	if (g_flags() & V4L2_BUF_FLAG_ERROR)
-		buf_states++;
 	if (g_flags() & V4L2_BUF_FLAG_PREPARED)
 		buf_states++;
 	fail_on_test(buf_states > 1);
 	fail_on_test(buf.length == 0);
-	if (is_planar()) {
+	if (v4l_type_is_planar(g_type())) {
 		fail_on_test(buf.length > VIDEO_MAX_PLANES);
 		for (unsigned p = 0; p < buf.length; p++) {
 			struct v4l2_plane *vp = buf.m.planes + p;
@@ -292,7 +267,7 @@ int buffer::check(unsigned type, unsigned memory, unsigned index,
 		}
 	}
 
-	if (is_capture() && !ts_copy &&
+	if (v4l_type_is_capture(g_type()) && !ts_copy &&
 	    (g_flags() & V4L2_BUF_FLAG_TIMECODE))
 		warn("V4L2_BUF_FLAG_TIMECODE was used!\n");
 
@@ -304,10 +279,10 @@ int buffer::check(unsigned type, unsigned memory, unsigned index,
 		}
 		fail_on_test(!g_timestamp().tv_sec && !g_timestamp().tv_usec);
 		fail_on_test(!(g_flags() & (V4L2_BUF_FLAG_DONE | V4L2_BUF_FLAG_ERROR)));
-		if (is_video()) {
+		if (v4l_type_is_video(g_type())) {
 			fail_on_test(g_field() == V4L2_FIELD_ALTERNATE);
 			fail_on_test(g_field() == V4L2_FIELD_ANY);
-			if (v4l_format_g_field(&cur_fmt) == V4L2_FIELD_ALTERNATE) {
+			if (cur_fmt.g_field() == V4L2_FIELD_ALTERNATE) {
 				fail_on_test(g_field() != V4L2_FIELD_BOTTOM &&
 						g_field() != V4L2_FIELD_TOP);
 				fail_on_test(g_field() == seq.last_field);
@@ -317,7 +292,7 @@ int buffer::check(unsigned type, unsigned memory, unsigned index,
 				else
 					fail_on_test((int)g_sequence() != seq.last_seq + 1);
 			} else {
-				fail_on_test(g_field() != v4l_format_g_field(&cur_fmt));
+				fail_on_test(g_field() != cur_fmt.g_field());
 				fail_on_test((int)g_sequence() != seq.last_seq + 1);
 			}
 		} else {
@@ -327,13 +302,12 @@ int buffer::check(unsigned type, unsigned memory, unsigned index,
 		seq.last_field = g_field();
 	} else {
 		fail_on_test(g_sequence());
-		if (mode == Queued && ts_copy && is_output()) {
+		if (mode == Queued && ts_copy && v4l_type_is_output(g_type())) {
 			fail_on_test(!g_timestamp().tv_sec && !g_timestamp().tv_usec);
 		} else {
 			fail_on_test(g_timestamp().tv_sec || g_timestamp().tv_usec);
-			fail_on_test(g_flags() & V4L2_BUF_FLAG_TIMECODE);
 		}
-		if (!is_output() || mode == Unqueued)
+		if (!v4l_type_is_output(g_type()) || mode == Unqueued)
 			fail_on_test(frame_types);
 		if (mode == Unqueued)
 			fail_on_test(g_flags() & (V4L2_BUF_FLAG_QUEUED | V4L2_BUF_FLAG_PREPARED |
@@ -356,7 +330,7 @@ static int testQueryBuf(struct node *node, unsigned type, unsigned count)
 
 	for (i = 0; i < count; i++) {
 		fail_on_test(buf.querybuf(node, i));
-		if (buf.is_planar())
+		if (v4l_type_is_planar(buf.g_type()))
 			fail_on_test(buf.buf.m.planes != buf.planes);
 		fail_on_test(buf.check(Unqueued, i));
 	}
@@ -365,21 +339,37 @@ static int testQueryBuf(struct node *node, unsigned type, unsigned count)
 	return 0;
 }
 
+static int testSetupVbi(struct node *node, int type)
+{
+	if (!v4l_type_is_vbi(type))
+		return 0;
+
+	if (!(node->cur_io_caps & V4L2_IN_CAP_STD))
+		return -1;
+
+	node->s_type(type);
+	cv4l_fmt vbi_fmt;
+
+	if (!node->g_fmt(vbi_fmt))
+		node->s_fmt(vbi_fmt);
+	return 0;
+}
+
 int testReqBufs(struct node *node)
 {
-	bool can_stream = node->caps & V4L2_CAP_STREAMING;
-	bool can_rw = node->caps & V4L2_CAP_READWRITE;
+	bool can_stream = node->g_caps() & V4L2_CAP_STREAMING;
+	bool can_rw = node->g_caps() & V4L2_CAP_READWRITE;
 	bool mmap_valid;
 	bool userptr_valid;
 	bool dmabuf_valid;
 	int ret;
 	unsigned i, m;
 	
-	reopen(node);
+	node->reopen();
 
-	queue q(node, 0, 0);
+	cv4l_queue q(0, 0);
 
-	ret = q.reqbufs(0);
+	ret = q.reqbufs(node, 0);
 	if (ret == ENOTTY) {
 		fail_on_test(can_stream);
 		return ret;
@@ -387,28 +377,32 @@ int testReqBufs(struct node *node)
 	fail_on_test(ret != EINVAL);
 	fail_on_test(node->node2 == NULL);
 	for (i = 1; i <= V4L2_BUF_TYPE_SDR_CAPTURE; i++) {
-		bool is_overlay = v4l_buf_type_is_overlay(i);
+		bool is_overlay = v4l_type_is_overlay(i);
 
 		if (!(node->valid_buftypes & (1 << i)))
 			continue;
+
+		if (testSetupVbi(node, i))
+			continue;
+
 		info("test buftype %s\n", buftype2s(i).c_str());
 		if (node->valid_buftype == 0)
 			node->valid_buftype = i;
 
 		q.init(0, 0);
-		fail_on_test(q.reqbufs(i) != EINVAL);
+		fail_on_test(q.reqbufs(node, i) != EINVAL);
 		q.init(i, V4L2_MEMORY_MMAP);
-		ret = q.reqbufs(0);
+		ret = q.reqbufs(node, 0);
 		fail_on_test(ret && ret != EINVAL);
 		mmap_valid = !ret;
 
 		q.init(i, V4L2_MEMORY_USERPTR);
-		ret = q.reqbufs(0);
+		ret = q.reqbufs(node, 0);
 		fail_on_test(ret && ret != EINVAL);
 		userptr_valid = !ret;
 
 		q.init(i, V4L2_MEMORY_DMABUF);
-		ret = q.reqbufs(0);
+		ret = q.reqbufs(node, 0);
 		fail_on_test(ret && ret != EINVAL);
 		dmabuf_valid = !ret;
 		fail_on_test((can_stream && !is_overlay) && !mmap_valid && !userptr_valid && !dmabuf_valid);
@@ -418,33 +412,33 @@ int testReqBufs(struct node *node)
 
 		if (mmap_valid) {
 			q.init(i, V4L2_MEMORY_MMAP);
-			fail_on_test(q.reqbufs(1));
+			fail_on_test(q.reqbufs(node, 1));
 			fail_on_test(q.g_buffers() == 0);
 			fail_on_test(q.g_memory() != V4L2_MEMORY_MMAP);
 			fail_on_test(q.g_type() != i);
-			fail_on_test(q.reqbufs(1));
+			fail_on_test(q.reqbufs(node, 1));
 			fail_on_test(testQueryBuf(node, i, q.g_buffers()));
 			node->valid_memorytype |= 1 << V4L2_MEMORY_MMAP;
 		}
 
 		if (userptr_valid) {
 			q.init(i, V4L2_MEMORY_USERPTR);
-			fail_on_test(q.reqbufs(1));
+			fail_on_test(q.reqbufs(node, 1));
 			fail_on_test(q.g_buffers() == 0);
 			fail_on_test(q.g_memory() != V4L2_MEMORY_USERPTR);
 			fail_on_test(q.g_type() != i);
-			fail_on_test(q.reqbufs(1));
+			fail_on_test(q.reqbufs(node, 1));
 			fail_on_test(testQueryBuf(node, i, q.g_buffers()));
 			node->valid_memorytype |= 1 << V4L2_MEMORY_USERPTR;
 		}
 
 		if (dmabuf_valid) {
 			q.init(i, V4L2_MEMORY_DMABUF);
-			fail_on_test(q.reqbufs(1));
+			fail_on_test(q.reqbufs(node, 1));
 			fail_on_test(q.g_buffers() == 0);
 			fail_on_test(q.g_memory() != V4L2_MEMORY_DMABUF);
 			fail_on_test(q.g_type() != i);
-			fail_on_test(q.reqbufs(1));
+			fail_on_test(q.reqbufs(node, 1));
 			fail_on_test(testQueryBuf(node, i, q.g_buffers()));
 			node->valid_memorytype |= 1 << V4L2_MEMORY_DMABUF;
 		}
@@ -453,30 +447,30 @@ int testReqBufs(struct node *node)
 			char buf = 0;
 
 			if (node->can_capture)
-				ret = test_read(node->vfd.fd, &buf, 1);
+				ret = node->read(&buf, 1);
 			else
-				ret = test_write(node->vfd.fd, &buf, 1);
+				ret = node->write(&buf, 1);
 			if (ret != -1)
 				return fail("Expected -1, got %d\n", ret);
 			if (errno != EBUSY)
 				return fail("Expected EBUSY, got %d\n", errno);
 		}
-		fail_on_test(q.reqbufs(0));
+		fail_on_test(q.reqbufs(node, 0));
 
 		for (m = V4L2_MEMORY_MMAP; m <= V4L2_MEMORY_DMABUF; m++) {
 			if (!(node->valid_memorytype & (1 << m)))
 				continue;
-			queue q2(node->node2, i, m);
-			fail_on_test(q.reqbufs(1));
+			cv4l_queue q2(i, m);
+			fail_on_test(q.reqbufs(node, 1));
 			if (!node->is_m2m) {
-				fail_on_test(q2.reqbufs(1) != EBUSY);
-				fail_on_test(q2.reqbufs() != EBUSY);
-				fail_on_test(q.reqbufs());
-				fail_on_test(q2.reqbufs(1));
-				fail_on_test(q2.reqbufs());
+				fail_on_test(q2.reqbufs(node->node2, 1) != EBUSY);
+				fail_on_test(q2.reqbufs(node->node2) != EBUSY);
+				fail_on_test(q.reqbufs(node));
+				fail_on_test(q2.reqbufs(node->node2, 1));
+				fail_on_test(q2.reqbufs(node->node2));
 			}
-			fail_on_test(q.reqbufs());
-			ret = q.create_bufs(1);
+			fail_on_test(q.reqbufs(node));
+			ret = q.create_bufs(node, 1);
 			if (ret == ENOTTY) {
 				warn("VIDIOC_CREATE_BUFS not supported\n");
 				break;
@@ -485,12 +479,12 @@ int testReqBufs(struct node *node)
 			fail_on_test(q.g_buffers() == 0);
 			fail_on_test(q.g_type() != i);
 			fail_on_test(testQueryBuf(node, i, q.g_buffers()));
-			fail_on_test(q.create_bufs(1));
+			fail_on_test(q.create_bufs(node, 1));
 			fail_on_test(testQueryBuf(node, i, q.g_buffers()));
 			if (!node->is_m2m)
-				fail_on_test(q2.create_bufs(1) != EBUSY);
+				fail_on_test(q2.create_bufs(node->node2, 1) != EBUSY);
 		}
-		fail_on_test(q.reqbufs());
+		fail_on_test(q.reqbufs(node));
 	}
 	return 0;
 }
@@ -501,45 +495,53 @@ int testExpBuf(struct node *node)
 	int type;
 
 	if (!(node->valid_memorytype & (1 << V4L2_MEMORY_MMAP))) {
-		queue q(node);
+		cv4l_queue q;
 
-		fail_on_test(q.has_expbuf());
+		fail_on_test(q.has_expbuf(node));
 		return ENOTTY;
 	}
 
 	for (type = 0; type <= V4L2_BUF_TYPE_SDR_CAPTURE; type++) {
 		if (!(node->valid_buftypes & (1 << type)))
 			continue;
-		if (v4l_buf_type_is_overlay(type))
+		if (v4l_type_is_overlay(type))
 			continue;
 
-		queue q(node, type, V4L2_MEMORY_MMAP);
+		if (testSetupVbi(node, type))
+			continue;
 
-		fail_on_test(q.reqbufs(1));
-		if (q.has_expbuf()) {
-			fail_on_test(q.export_bufs());
+		cv4l_queue q(type, V4L2_MEMORY_MMAP);
+
+		fail_on_test(q.reqbufs(node, 1));
+		if (q.has_expbuf(node)) {
+			fail_on_test(q.export_bufs(node));
 			have_expbuf = true;
 		} else {
-			fail_on_test(!q.export_bufs());
+			fail_on_test(!q.export_bufs(node));
 		}
 		q.close_exported_fds();
-		fail_on_test(q.reqbufs());
+		fail_on_test(q.reqbufs(node));
 	}
 	return have_expbuf ? 0 : ENOTTY;
 }
 
 int testReadWrite(struct node *node)
 {
-	bool can_rw = node->caps & V4L2_CAP_READWRITE;
-	int fd_flags = fcntl(node->vfd.fd, F_GETFL);
+	bool can_rw = node->g_caps() & V4L2_CAP_READWRITE;
+	int fd_flags = fcntl(node->g_fd(), F_GETFL);
 	char buf = 0;
 	int ret;
 
-	fcntl(node->vfd.fd, F_SETFL, fd_flags | O_NONBLOCK);
+	if (v4l_has_vbi(node->g_v4l_fd()) &&
+	    !(node->cur_io_caps & V4L2_IN_CAP_STD)) {
+		return 0;
+	}
+
+	fcntl(node->g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
 	if (node->can_capture)
-		ret = read(node->vfd.fd, &buf, 1);
+		ret = node->read(&buf, 1);
 	else
-		ret = write(node->vfd.fd, &buf, 1);
+		ret = node->write(&buf, 1);
 	// Note: RDS can only return multiples of 3, so we accept
 	// both 0 and 1 as return code.
 	// EBUSY can be returned when attempting to read/write to a
@@ -551,22 +553,22 @@ int testReadWrite(struct node *node)
 	if (!can_rw)
 		return ENOTTY;
 
-	reopen(node);
-	fcntl(node->vfd.fd, F_SETFL, fd_flags | O_NONBLOCK);
+	node->reopen();
+	fcntl(node->g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
 
 	/* check that the close cleared the busy flag */
 	if (node->can_capture)
-		ret = read(node->vfd.fd, &buf, 1);
+		ret = node->read(&buf, 1);
 	else
-		ret = write(node->vfd.fd, &buf, 1);
+		ret = node->write(&buf, 1);
 	fail_on_test((ret < 0 && errno != EAGAIN && errno != EBUSY) || ret > 1);
 	return 0;
 }
 
-static int captureBufs(struct node *node, const queue &q,
-		const queue &m2m_q, unsigned frame_count, bool use_poll)
+static int captureBufs(struct node *node, const cv4l_queue &q,
+		const cv4l_queue &m2m_q, unsigned frame_count, bool use_poll)
 {
-	int fd_flags = fcntl(node->vfd.fd, F_GETFL);
+	int fd_flags = fcntl(node->g_fd(), F_GETFL);
 	buffer buf(q);
 	unsigned count = frame_count;
 	int ret;
@@ -578,7 +580,7 @@ static int captureBufs(struct node *node, const queue &q,
 	}
 
 	if (use_poll)
-		fcntl(node->vfd.fd, F_SETFL, fd_flags | O_NONBLOCK);
+		fcntl(node->g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
 	for (;;) {
 		buf.init(q);
 
@@ -587,18 +589,18 @@ static int captureBufs(struct node *node, const queue &q,
 			fd_set fds;
 
 			FD_ZERO(&fds);
-			FD_SET(node->vfd.fd, &fds);
+			FD_SET(node->g_fd(), &fds);
 			if (node->is_m2m)
-				ret = select(node->vfd.fd + 1, &fds, &fds, NULL, &tv);
-			else if (q.is_output())
-				ret = select(node->vfd.fd + 1, NULL, &fds, NULL, &tv);
+				ret = select(node->g_fd() + 1, &fds, &fds, NULL, &tv);
+			else if (v4l_type_is_output(q.g_type()))
+				ret = select(node->g_fd() + 1, NULL, &fds, NULL, &tv);
 			else
-				ret = select(node->vfd.fd + 1, &fds, NULL, NULL, &tv);
+				ret = select(node->g_fd() + 1, &fds, NULL, NULL, &tv);
 			fail_on_test(ret <= 0);
-			fail_on_test(!FD_ISSET(node->vfd.fd, &fds));
+			fail_on_test(!FD_ISSET(node->g_fd(), &fds));
 		}
 
-		ret = buf.dqbuf(q);
+		ret = buf.dqbuf(node);
 		if (ret != EAGAIN) {
 			fail_on_test(ret);
 			if (show_info)
@@ -610,9 +612,11 @@ static int captureBufs(struct node *node, const queue &q,
 				printf("\r\t%s: Frame #%03d%s",
 						buftype2s(q.g_type()).c_str(),
 						frame_count - count, use_poll ? " (polling)" : "");
+				if (node->g_trace())
+					printf("\n");
 				fflush(stdout);
 			}
-			if (buf.is_capture() && node->is_m2m && buf.ts_is_copy()) {
+			if (v4l_type_is_capture(buf.g_type()) && node->is_m2m && buf.ts_is_copy()) {
 				fail_on_test(buffer_info.find(buf.g_timestamp()) == buffer_info.end());
 				struct v4l2_buffer &orig_buf = buffer_info[buf.g_timestamp()];
 				fail_on_test(buf.g_field() != orig_buf.field);
@@ -622,7 +626,7 @@ static int captureBufs(struct node *node, const queue &q,
 					fail_on_test(memcmp(&buf.g_timecode(), &orig_buf.timecode,
 								sizeof(orig_buf.timecode)));
 			}
-			fail_on_test(buf.qbuf(q));
+			fail_on_test(buf.qbuf(node));
 			if (--count == 0)
 				break;
 		}
@@ -630,7 +634,7 @@ static int captureBufs(struct node *node, const queue &q,
 			continue;
 
 		buf.init(m2m_q);
-		ret = buf.dqbuf(m2m_q);
+		ret = buf.dqbuf(node);
 		if (ret == EAGAIN)
 			continue;
 		if (show_info)
@@ -639,7 +643,7 @@ static int captureBufs(struct node *node, const queue &q,
 				buf.g_timestamp().tv_sec, buf.g_timestamp().tv_usec);
 		fail_on_test(ret);
 		fail_on_test(buf.check(m2m_q, last_m2m_seq));
-		if (buf.is_capture() && buf.ts_is_copy()) {
+		if (v4l_type_is_capture(buf.g_type()) && buf.ts_is_copy()) {
 			fail_on_test(buffer_info.find(buf.g_timestamp()) == buffer_info.end());
 			struct v4l2_buffer &orig_buf = buffer_info[buf.g_timestamp()];
 			fail_on_test(buf.g_field() != orig_buf.field);
@@ -649,10 +653,10 @@ static int captureBufs(struct node *node, const queue &q,
 				fail_on_test(memcmp(&buf.g_timecode(), &orig_buf.timecode,
 							sizeof(orig_buf.timecode)));
 		}
-		fail_on_test(buf.qbuf(m2m_q));
+		fail_on_test(buf.qbuf(node));
 	}
 	if (use_poll)
-		fcntl(node->vfd.fd, F_SETFL, fd_flags);
+		fcntl(node->g_fd(), F_SETFL, fd_flags);
 	if (!show_info) {
 		printf("\r\t                                                  \r");
 		fflush(stdout);
@@ -662,33 +666,33 @@ static int captureBufs(struct node *node, const queue &q,
 
 static unsigned invert_buf_type(unsigned type)
 {
-	if (v4l_buf_type_is_planar(type))
-		return v4l_buf_type_is_output(type) ?
+	if (v4l_type_is_planar(type))
+		return v4l_type_is_output(type) ?
 			V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE :
 			V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
-	return v4l_buf_type_is_output(type) ?
+	return v4l_type_is_output(type) ?
 		V4L2_BUF_TYPE_VIDEO_CAPTURE :
 		V4L2_BUF_TYPE_VIDEO_OUTPUT;
 }
 
-static int setupM2M(struct node *node, queue &q)
+static int setupM2M(struct node *node, cv4l_queue &q)
 {
 	last_m2m_seq.init();
 
-	fail_on_test(q.reqbufs(1));
+	fail_on_test(q.reqbufs(node, 1));
 	for (unsigned i = 0; i < q.g_buffers(); i++) {
 		buffer buf(q);
 
-		fail_on_test(buf.querybuf(q, i));
-		fail_on_test(buf.qbuf(q));
+		fail_on_test(buf.querybuf(node, i));
+		fail_on_test(buf.qbuf(node));
 	}
-	if (q.is_video()) {
-		v4l2_format fmt;
+	if (v4l_type_is_video(q.g_type())) {
+		cv4l_fmt fmt(q.g_type());
 
-		v4l_g_fmt(&node->vfd, &fmt, q.g_type());
-		last_m2m_seq.last_field = v4l_format_g_field(&fmt);
+		node->g_fmt(fmt);
+		last_m2m_seq.last_field = fmt.g_field();
 	}
-	fail_on_test(q.streamon());
+	fail_on_test(node->streamon(q.g_type()));
 	return 0;
 }
 
@@ -707,7 +711,7 @@ static int bufferOutputErrorTest(struct node *node, const buffer &orig_buf)
 	have_prepare = ret != ENOTTY;
 	fail_on_test(buf.qbuf(node, false) != EINVAL);
 
-	if (buf.is_planar()) {
+	if (v4l_type_is_planar(buf.g_type())) {
 		for (unsigned p = 0; p < buf.g_num_planes(); p++) {
 			buf.s_bytesused(buf.g_length(p) / 2, p);
 			buf.s_data_offset(buf.g_bytesused(p), p);
@@ -738,56 +742,56 @@ static int bufferOutputErrorTest(struct node *node, const buffer &orig_buf)
 	return 0;
 }
 
-static int setupMmap(struct node *node, queue &q)
+static int setupMmap(struct node *node, cv4l_queue &q)
 {
 	for (unsigned i = 0; i < q.g_buffers(); i++) {
 		buffer buf(q);
 		int ret;
 
-		fail_on_test(buf.querybuf(q, i));
+		fail_on_test(buf.querybuf(node, i));
 		fail_on_test(buf.check(q, Unqueued, i));
 
 		for (unsigned p = 0; p < buf.g_num_planes(); p++) {
 			// Try a random offset
-			fail_on_test(test_mmap(NULL, buf.g_length(p), PROT_READ | PROT_WRITE,
-				MAP_SHARED, node->vfd.fd, buf.g_mem_offset(p) + 0xdeadbeef) != MAP_FAILED);
+			fail_on_test(node->mmap(buf.g_length(p),
+				buf.g_mem_offset(p) + 0xdeadbeef) != MAP_FAILED);
 		}
-		fail_on_test(!buf.dqbuf(q));
-		if (buf.is_output() && i == 0) {
+		fail_on_test(!buf.dqbuf(node));
+		if (v4l_type_is_output(buf.g_type()) && i == 0) {
 			fail_on_test(bufferOutputErrorTest(node, buf));
-			fail_on_test(buf.querybuf(q, i));
+			fail_on_test(buf.querybuf(node, i));
 			fail_on_test(buf.check(q, Queued, i));
 		} else {
-			ret = buf.prepare_buf(q);
+			ret = buf.prepare_buf(node);
 			fail_on_test(ret && ret != ENOTTY);
 			if (ret == 0) {
-				fail_on_test(buf.querybuf(q, i));
+				fail_on_test(buf.querybuf(node, i));
 				fail_on_test(buf.check(q, Prepared, i));
-				fail_on_test(!buf.prepare_buf(q));
+				fail_on_test(!buf.prepare_buf(node));
 			}
 
-			fail_on_test(buf.qbuf(q));
-			fail_on_test(!buf.qbuf(q));
-			fail_on_test(!buf.prepare_buf(q));
+			fail_on_test(buf.qbuf(node));
+			fail_on_test(!buf.qbuf(node));
+			fail_on_test(!buf.prepare_buf(node));
 			// Test with invalid buffer index
 			buf.s_index(buf.g_index() + VIDEO_MAX_FRAME);
-			fail_on_test(!buf.prepare_buf(q));
-			fail_on_test(!buf.qbuf(q));
-			fail_on_test(!buf.querybuf(q, buf.g_index()));
+			fail_on_test(!buf.prepare_buf(node));
+			fail_on_test(!buf.qbuf(node));
+			fail_on_test(!buf.querybuf(node, buf.g_index()));
 			buf.s_index(buf.g_index() - VIDEO_MAX_FRAME);
 			fail_on_test(buf.g_index() != i);
 		}
-		fail_on_test(buf.querybuf(q, i));
+		fail_on_test(buf.querybuf(node, i));
 		fail_on_test(buf.check(q, Queued, i));
-		fail_on_test(!buf.dqbuf(q));
+		fail_on_test(!buf.dqbuf(node));
 	}
-	fail_on_test(q.mmap_bufs());
+	fail_on_test(q.mmap_bufs(node));
 	return 0;
 }
 
 int testMmap(struct node *node, unsigned frame_count)
 {
-	bool can_stream = node->caps & V4L2_CAP_STREAMING;
+	bool can_stream = node->g_caps() & V4L2_CAP_STREAMING;
 	bool have_createbufs = true;
 	int type;
 	int ret;
@@ -799,102 +803,107 @@ int testMmap(struct node *node, unsigned frame_count)
 	for (type = 0; type <= V4L2_BUF_TYPE_SDR_CAPTURE; type++) {
 		if (!(node->valid_buftypes & (1 << type)))
 			continue;
-		if (v4l_buf_type_is_overlay(type))
+		if (v4l_type_is_overlay(type))
 			continue;
 
-		queue q(node, type, V4L2_MEMORY_MMAP);
-		queue m2m_q(node, invert_buf_type(type));
+		cv4l_queue q(type, V4L2_MEMORY_MMAP);
+		cv4l_queue m2m_q(invert_buf_type(type));
+	
+		if (testSetupVbi(node, type))
+			continue;
 
-		ret = q.reqbufs(0);
+		ret = q.reqbufs(node, 0);
 		if (ret) {
 			fail_on_test(can_stream);
 			return ret;
 		}
 		fail_on_test(!can_stream);
 
-		fail_on_test(q.streamon() != EINVAL);
-		fail_on_test(q.streamoff());
+		fail_on_test(node->streamon(q.g_type()) != EINVAL);
+		fail_on_test(node->streamoff(q.g_type()));
 
 		q.init(type, V4L2_MEMORY_MMAP);
-		fail_on_test(q.reqbufs(1));
-		fail_on_test(q.streamoff());
+		fail_on_test(q.reqbufs(node, 1));
+		fail_on_test(node->streamoff(q.g_type()));
 		last_seq.init();
 
 		// Test queuing buffers...
 		for (unsigned i = 0; i < q.g_buffers(); i++) {
 			buffer buf(q);
 
-			fail_on_test(buf.querybuf(q, i));
-			fail_on_test(buf.qbuf(q));
+			fail_on_test(buf.querybuf(node, i));
+			fail_on_test(buf.qbuf(node));
 		}
 		// calling STREAMOFF...
-		fail_on_test(q.streamoff());
+		fail_on_test(node->streamoff(q.g_type()));
 		// and now we should be able to queue those buffers again since
 		// STREAMOFF should return them back to the dequeued state.
 		for (unsigned i = 0; i < q.g_buffers(); i++) {
 			buffer buf(q);
 
-			fail_on_test(buf.querybuf(q, i));
-			fail_on_test(buf.qbuf(q));
+			fail_on_test(buf.querybuf(node, i));
+			fail_on_test(buf.qbuf(node));
 		}
 		// Now request buffers again, freeing the old buffers.
 		// Good check for whether all the internal vb2 calls are in
 		// balance.
-		fail_on_test(q.reqbufs(q.g_buffers()));
-		v4l_g_fmt(&node->vfd, &cur_fmt, q.g_type());
+		fail_on_test(q.reqbufs(node, q.g_buffers()));
+		cur_fmt.s_type(q.g_type());
+		node->g_fmt(cur_fmt);
 
-		ret = q.create_bufs(0);
+		ret = q.create_bufs(node, 0);
 		fail_on_test(ret != ENOTTY && ret != 0);
 		if (ret == ENOTTY)
 			have_createbufs = false;
 		if (have_createbufs) {
-			v4l2_format fmt = cur_fmt;
+			cv4l_fmt fmt(cur_fmt);
 
 			if (node->is_video) {
-				last_seq.last_field = v4l_format_g_field(&cur_fmt);
-				v4l_format_s_height(&fmt, v4l_format_g_height(&fmt) / 2);
-				for (unsigned p = 0; p < v4l_format_g_num_planes(&fmt); p++)
-					v4l_format_s_sizeimage(&fmt, p, v4l_format_g_sizeimage(&fmt, p) / 2);
-				ret = q.create_bufs(1, &fmt);
+				last_seq.last_field = cur_fmt.g_field();
+				fmt.s_height(fmt.g_height() / 2);
+				for (unsigned p = 0; p < fmt.g_num_planes(); p++)
+					fmt.s_sizeimage(fmt.g_sizeimage(p) / 2, p);
+				ret = q.create_bufs(node, 1, &fmt);
 				fail_on_test(ret != EINVAL);
 				fail_on_test(testQueryBuf(node, cur_fmt.type, q.g_buffers()));
 				fmt = cur_fmt;
-				for (unsigned p = 0; p < v4l_format_g_num_planes(&fmt); p++)
-					v4l_format_s_sizeimage(&fmt, p, v4l_format_g_sizeimage(&fmt, p) * 2);
+				for (unsigned p = 0; p < fmt.g_num_planes(); p++)
+					fmt.s_sizeimage(fmt.g_sizeimage(p) * 2, p);
 			}
-			fail_on_test(q.create_bufs(1, &fmt));
+			fail_on_test(q.create_bufs(node, 1, &fmt));
 		}
 		fail_on_test(setupMmap(node, q));
 
-		fail_on_test(q.streamon());
-		fail_on_test(q.streamon());
+		fail_on_test(node->streamon(q.g_type()));
+		fail_on_test(node->streamon(q.g_type()));
 
 		if (node->is_m2m)
 			fail_on_test(setupM2M(node, m2m_q));
 		else
 			fail_on_test(captureBufs(node, q, m2m_q, frame_count, false));
 		fail_on_test(captureBufs(node, q, m2m_q, frame_count, true));
-		fail_on_test(q.streamoff());
-		fail_on_test(q.streamoff());
+		fail_on_test(node->streamoff(q.g_type()));
+		fail_on_test(node->streamoff(q.g_type()));
+		q.munmap_bufs(node);
+		fail_on_test(q.reqbufs(node, 0));
 	}
 	return 0;
 }
 
-static int setupUserPtr(struct node *node, queue &q)
+static int setupUserPtr(struct node *node, cv4l_queue &q)
 {
-	fail_on_test(q.alloc_bufs());
+	fail_on_test(q.alloc_bufs(node));
 
 	for (unsigned i = 0; i < q.g_buffers(); i++) {
 		buffer buf(q);
 		int ret;
 
-		fail_on_test(buf.querybuf(q, i));
+		fail_on_test(buf.querybuf(node, i));
 		fail_on_test(buf.check(q, Unqueued, i));
 
 		for (unsigned p = 0; p < buf.g_num_planes(); p++) {
 			// This should not work!
-			fail_on_test(test_mmap(NULL, buf.g_length(p), PROT_READ | PROT_WRITE,
-					MAP_SHARED, node->vfd.fd, 0) != MAP_FAILED);
+			fail_on_test(node->mmap(buf.g_length(p), 0) != MAP_FAILED);
 		}
 
 		ret = ENOTTY;
@@ -902,39 +911,39 @@ static int setupUserPtr(struct node *node, queue &q)
 		if ((i & 1) == 0) {
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
 				buf.s_userptr(0UL, p);
-			ret = buf.prepare_buf(q);
+			ret = buf.prepare_buf(node);
 			fail_on_test(!ret);
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
 				buf.s_userptr((char *)q.g_userptr(i, p) + 0xdeadbeef, p);
-			ret = buf.prepare_buf(q);
+			ret = buf.prepare_buf(node);
 			fail_on_test(!ret);
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
 				buf.s_userptr(q.g_userptr(i, p), p);
-			ret = buf.prepare_buf(q);
+			ret = buf.prepare_buf(node);
 			fail_on_test(ret && ret != ENOTTY);
 
 			if (ret == 0) {
-				fail_on_test(buf.querybuf(q, i));
+				fail_on_test(buf.querybuf(node, i));
 				fail_on_test(buf.check(q, Prepared, i));
 			}
 		}
 		if (ret == ENOTTY) {
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
 				buf.s_userptr(0UL, p);
-			ret = buf.qbuf(q);
+			ret = buf.qbuf(node);
 			fail_on_test(!ret);
 
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
 				buf.s_userptr((char *)q.g_userptr(i, p) + 0xdeadbeef, p);
-			ret = buf.qbuf(q);
+			ret = buf.qbuf(node);
 			fail_on_test(!ret);
 
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
 				buf.s_userptr(q.g_userptr(i, p), p);
 		}
 
-		fail_on_test(buf.qbuf(q));
-		fail_on_test(buf.querybuf(q, i));
+		fail_on_test(buf.qbuf(node));
+		fail_on_test(buf.querybuf(node, i));
 		fail_on_test(buf.check(q, Queued, i));
 	}
 	return 0;
@@ -942,7 +951,7 @@ static int setupUserPtr(struct node *node, queue &q)
 
 int testUserPtr(struct node *node, unsigned frame_count)
 {
-	bool can_stream = node->caps & V4L2_CAP_STREAMING;
+	bool can_stream = node->g_caps() & V4L2_CAP_STREAMING;
 	int type;
 	int ret;
 
@@ -953,13 +962,16 @@ int testUserPtr(struct node *node, unsigned frame_count)
 	for (type = 0; type <= V4L2_BUF_TYPE_SDR_CAPTURE; type++) {
 		if (!(node->valid_buftypes & (1 << type)))
 			continue;
-		if (v4l_buf_type_is_overlay(type))
+		if (v4l_type_is_overlay(type))
 			continue;
 
-		queue q(node, type, V4L2_MEMORY_USERPTR);
-		queue m2m_q(node, invert_buf_type(type));
+		cv4l_queue q(type, V4L2_MEMORY_USERPTR);
+		cv4l_queue m2m_q(invert_buf_type(type));
 
-		ret = q.reqbufs(0);
+		if (testSetupVbi(node, type))
+			continue;
+
+		ret = q.reqbufs(node, 0);
 		if (ret) {
 			fail_on_test(ret != EINVAL);
 			return ENOTTY;
@@ -967,76 +979,75 @@ int testUserPtr(struct node *node, unsigned frame_count)
 		fail_on_test(!can_stream);
 
 		q.init(type, V4L2_MEMORY_USERPTR);
-		fail_on_test(q.reqbufs(1));
-		fail_on_test(q.streamoff());
+		fail_on_test(q.reqbufs(node, 1));
+		fail_on_test(node->streamoff(q.g_type()));
 		last_seq.init();
 		if (node->is_video)
-			last_seq.last_field = v4l_format_g_field(&cur_fmt);
+			last_seq.last_field = cur_fmt.g_field();
 
 		fail_on_test(setupUserPtr(node, q));
 
-		fail_on_test(q.streamon());
-		fail_on_test(q.streamon());
+		fail_on_test(node->streamon(q.g_type()));
+		fail_on_test(node->streamon(q.g_type()));
 
 		if (node->is_m2m)
 			fail_on_test(setupM2M(node, m2m_q));
 		else
 			fail_on_test(captureBufs(node, q, m2m_q, frame_count, false));
 		fail_on_test(captureBufs(node, q, m2m_q, frame_count, true));
-		fail_on_test(q.streamoff());
-		fail_on_test(q.streamoff());
+		fail_on_test(node->streamoff(q.g_type()));
+		fail_on_test(node->streamoff(q.g_type()));
 	}
 	return 0;
 }
 
 static int setupDmaBuf(struct node *expbuf_node, struct node *node,
-		       queue &q, queue &exp_q)
+		       cv4l_queue &q, cv4l_queue &exp_q)
 {
-	fail_on_test(exp_q.reqbufs(q.g_buffers()));
+	fail_on_test(exp_q.reqbufs(expbuf_node, q.g_buffers()));
 	fail_on_test(exp_q.g_buffers() < q.g_buffers());
-	fail_on_test(exp_q.export_bufs());
+	fail_on_test(exp_q.export_bufs(expbuf_node));
 
 	for (unsigned i = 0; i < q.g_buffers(); i++) {
 		buffer buf(q);
 		int ret;
 
-		fail_on_test(buf.querybuf(q, i));
+		fail_on_test(buf.querybuf(node, i));
 		fail_on_test(buf.check(q, Unqueued, i));
 		fail_on_test(exp_q.g_num_planes() < buf.g_num_planes());
 		for (unsigned p = 0; p < buf.g_num_planes(); p++) {
 			fail_on_test(exp_q.g_length(p) < buf.g_length(p));
 			// This should not work!
-			fail_on_test(test_mmap(NULL, buf.g_length(p), PROT_READ | PROT_WRITE,
-						MAP_SHARED, node->vfd.fd, 0) != MAP_FAILED);
+			fail_on_test(node->mmap(buf.g_length(p), 0) != MAP_FAILED);
 			q.s_fd(i, p, exp_q.g_fd(i, p));
 		}
 
 		for (unsigned p = 0; p < buf.g_num_planes(); p++)
 			buf.s_fd(0xdeadbeef + q.g_fd(i, p), p);
-		ret = buf.prepare_buf(q);
+		ret = buf.prepare_buf(node);
 		fail_on_test(!ret);
 		if (ret != ENOTTY) {
 			buf.init(q, i);
-			ret = buf.prepare_buf(q);
+			ret = buf.prepare_buf(node);
 			fail_on_test(ret);
-			fail_on_test(buf.querybuf(q, i));
+			fail_on_test(buf.querybuf(node, i));
 			fail_on_test(buf.check(q, Prepared, i));
 		} else {
-			fail_on_test(!buf.qbuf(q));
+			fail_on_test(!buf.qbuf(node));
 			buf.init(q, i);
 		}
 
-		fail_on_test(buf.qbuf(q));
-		fail_on_test(buf.querybuf(q, i));
+		fail_on_test(buf.qbuf(node));
+		fail_on_test(buf.querybuf(node, i));
 		fail_on_test(buf.check(q, Queued, i));
 	}
-	fail_on_test(q.mmap_bufs());
+	fail_on_test(q.mmap_bufs(node));
 	return 0;
 }
 
 int testDmaBuf(struct node *expbuf_node, struct node *node, unsigned frame_count)
 {
-	bool can_stream = node->caps & V4L2_CAP_STREAMING;
+	bool can_stream = node->g_caps() & V4L2_CAP_STREAMING;
 	int expbuf_type, type;
 	int ret;
 
@@ -1047,49 +1058,52 @@ int testDmaBuf(struct node *expbuf_node, struct node *node, unsigned frame_count
 	for (type = 0; type <= V4L2_BUF_TYPE_SDR_CAPTURE; type++) {
 		if (!(node->valid_buftypes & (1 << type)))
 			continue;
-		if (v4l_buf_type_is_sdr(type))
+		if (v4l_type_is_sdr(type))
 			continue;
-		if (v4l_buf_type_is_overlay(type))
+		if (v4l_type_is_overlay(type))
 			continue;
 
-		if (expbuf_node->caps & V4L2_CAP_VIDEO_CAPTURE_MPLANE)
+		if (expbuf_node->g_caps() & V4L2_CAP_VIDEO_CAPTURE_MPLANE)
 			expbuf_type = V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE;
-		else if (expbuf_node->caps & V4L2_CAP_VIDEO_CAPTURE)
+		else if (expbuf_node->g_caps() & V4L2_CAP_VIDEO_CAPTURE)
 			expbuf_type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-		else if (expbuf_node->caps & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
+		else if (expbuf_node->g_caps() & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
 			expbuf_type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
 		else 
 			expbuf_type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
 
-		queue q(node, type, V4L2_MEMORY_DMABUF);
-		queue m2m_q(node, invert_buf_type(type));
-		queue exp_q(expbuf_node, expbuf_type, V4L2_MEMORY_MMAP);
+		cv4l_queue q(type, V4L2_MEMORY_DMABUF);
+		cv4l_queue m2m_q(invert_buf_type(type));
+		cv4l_queue exp_q(expbuf_type, V4L2_MEMORY_MMAP);
 
-		ret = q.reqbufs(0);
+		if (testSetupVbi(node, type))
+			continue;
+
+		ret = q.reqbufs(node, 0);
 		if (ret) {
 			fail_on_test(ret != EINVAL);
 			return ENOTTY;
 		}
 		fail_on_test(!can_stream);
 
-		fail_on_test(q.reqbufs(1));
-		fail_on_test(q.streamoff());
+		fail_on_test(q.reqbufs(node, 1));
+		fail_on_test(node->streamoff(q.g_type()));
 		last_seq.init();
 		if (node->is_video)
-			last_seq.last_field = v4l_format_g_field(&cur_fmt);
+			last_seq.last_field = cur_fmt.g_field();
 
 		fail_on_test(setupDmaBuf(expbuf_node, node, q, exp_q));
 
-		fail_on_test(q.streamon());
-		fail_on_test(q.streamon());
+		fail_on_test(node->streamon(q.g_type()));
+		fail_on_test(node->streamon(q.g_type()));
 
 		if (node->is_m2m)
 			fail_on_test(setupM2M(node, m2m_q));
 		else
 			fail_on_test(captureBufs(node, q, m2m_q, frame_count, false));
 		fail_on_test(captureBufs(node, q, m2m_q, frame_count, true));
-		fail_on_test(q.streamoff());
-		fail_on_test(q.streamoff());
+		fail_on_test(node->streamoff(q.g_type()));
+		fail_on_test(node->streamoff(q.g_type()));
 	}
 	return 0;
 }
