@@ -13,11 +13,6 @@
 #include <sys/time.h>
 #include <dirent.h>
 #include <math.h>
-#include <config.h>
-
-#include <linux/videodev2.h>
-#include <libv4l2.h>
-#include <string>
 
 #include "v4l2-ctl.h"
 
@@ -139,6 +134,11 @@ static void parse_dv_bt_timings(char *optarg, struct v4l2_dv_timings *dv_timings
 
 	dv_timings->type = V4L2_DV_BT_656_1120;
 
+	if (!strcmp(subs, "query")) {
+		query = true;
+		return;
+	}
+
 	while (*subs != '\0') {
 		static const char *const subopts[] = {
 			"width",
@@ -156,7 +156,6 @@ static void parse_dv_bt_timings(char *optarg, struct v4l2_dv_timings *dv_timings
 			"il_vs",
 			"il_vbp",
 			"index",
-			"query",
 			NULL
 		};
 
@@ -206,9 +205,6 @@ static void parse_dv_bt_timings(char *optarg, struct v4l2_dv_timings *dv_timings
 		case 14:
 			enumerate = strtol(value, 0L, 0);
 			break;
-		case 15:
-			query = true;
-			break;
 		default:
 			stds_usage();
 			exit(1);
